##ERROR HANDLERS

Un error handler est une classe dont la logique va être executée lorsqu'une erreur intervient durant l'import. 

Ce error handler peut-être associé à exceptions spécifiques et il est possible de configurer le bubbling : le fait de faire remonter l'erreur ou non jusqu'aux logs. 

Un exemple d'usage du error handler est de détecter les clés dupliquées lors de l'insertion des données dans la base de donnée cible pour insérer un mapping : on détecte ainsi les doublons potentiels. 

###Créer un error handler

La création d'un error handler se fait en étandant la classe abstraite `AbstractErrorHandler`

Exemple :

```php
<?php

declare(strict_types=1);

namespace App\Service\ErrorHandler;

class SaveDuplicateHandler implements ErrorHandlerInterface
{
public function getType(): string
    {
        return 'save_duplicate';
    }

    public function execute(TableMappingDefinition $tableMappingDefinition, array $data): void
    {
        $configuration = $this->getConfiguration();
        $configurationKey = $this->getConfiguration('key');
        ...        
    }

    public function getErrorBubbling(): bool
    {
        return false;
    }

    public function can(Exception $exception): bool
    {
        return $exception instanceof UniqueConstraintViolationException;
    }

}
```

La valeur retournée par `getType` sera utilisée pour matcher la configuration `migration_mapping.yaml`. Pour utiliser ce error handler on utilisera la config suivante :

```yaml
migration_mapping:
  table:
    ...
  error_handlers:
      save_duplicate:
          columns: [ code ]
```

Chaque error handler déclaré dans la configuration peut recevoir un array clé / valeur qui sera accessible via la fonction `getConfiguration` lors de l'execution du handler

Pour déclarer un handler sans configuration, le mapping serait le suivant 


```yaml
migration_mapping:
  table:
    ...
  error_handlers:
      save_duplicate: ~
```

La fonction `can` permet de déterminer si l'erreur récupérée par la MigrationExecutor devrait être traitée ou non par le handler. Cela permet de binder plusieurs handlers à différentes exceptions. 

La fonction `getErrorBubbling` détermine si l'erreur traitée par le handler devrait être relancée une fois l'execution du error handler terminée. De cette manière, l'erreur peut être ignorée ou loggée. 

Les `$data` transmises à la fonction `execute` sont les data transformées, c'est à dire après application des transformers configurés dans le mapping.

###Error Handlers disponibles

####SaveDuplicateHandler
**type**: `save_duplicate`

**configuration**: columns - colonnes permettant de déduire la clé unique

Lorsqu'une `UniqueConstraintViolationException` intervient, ce error handler enregistre une ligne dans la table de mapping.

On identifie, grâce aux colonnes passées dans la clé de configuration `columns`, la ligne déjà existante en base de données et on enregistré les IDs correspondants.

Cela nous permet de mapper une deuxième ligne de la source sur une ligne déjà existante dans la base de données cible
